#include <syscall.h>
#include "selector.h"

#define INLINE static inline

#define CHECKERR "jnc    1f\n\tmovl   $-1, %0 \n\tjmp    2f \n\t1: \n\t"
#define CHECKERR0 "jnc    1f\n\tmovl   $0, %0 \n\tjmp    2f \n\t1: \n\t"
#define CHECK_ERR_DPMI10 "jc    2f\n\t"
#define OKEAX0 "xorl   %0, %0 \n\t2: \n\t"
#define END "2: \n\t"

/* return selector or null */
INLINE int AllocLDT(int n_sel)
{
    register int _v;

    __asm__ __volatile__(
	"call   _dpmi31 \n\t"
	CHECKERR0
	"movzwl %%ax, %0\n\t"
	END
	: "=a" (_v)
	: "c" (n_sel) , "0" (0x000)
	: "ax","cx" );
    return _v ;
}

INLINE int FreeLDT(int sel)
{
    register int _v;

    __asm__ __volatile__(
	"call   _dpmi31 \n\t"
        CHECKERR
	OKEAX0
	: "=a" (_v)
	: "b" (sel) , "0" (0x001)
	: "ax","bx" );

    return _v ;
}

/* return base address or null */
INLINE int GetBaseAddress(int sel)
{
    register int _v;

    __asm__ __volatile__(
	"call   _dpmi31 \n\t"
	CHECKERR0
	"shll   $16, %%ecx  \n\t"
	"movw   %%dx, %%cx \n\t"
	"movl   %%ecx, %0 \n\t"
	END
	: "=a" (_v)
	: "b" (sel) , "0" (0x006)
	: "ax","bx","cx","dx" );

    return _v ;
}

INLINE int SetBaseAddress(int sel,int base)
{
    register int _v;

    __asm__ __volatile__(
	"call   _dpmi31 \n\t"
        CHECKERR
	OKEAX0
	: "=a" (_v)
	: "b" (sel),"c" ((short)(base>>16)),"d" ((short)base),"0" (0x007)
	: "ax","bx","cx","dx" );

    return _v ;
}

INLINE int SetLimit(int sel,int limit)
{
    register int _v;

    __asm__ __volatile__(
	"call   _dpmi31 \n\t"
        CHECKERR
	OKEAX0
	: "=a" (_v)
	: "b" (sel),"c" ((short)(limit>>16)),"d" ((short)limit),"0" (0x008)
	: "ax","bx","cx","dx" );

    return _v ;
}

INLINE unsigned short get_data_seg(void)
{
    register unsigned short v;
    __asm__("mov %%ds, %0 \n\t"
	    : "=r" ((unsigned short) v) );
    return v;
}


int init_selector(int * sel)
{
    if (!(*sel = AllocLDT(1)))
	return -1;

    if (SetLimit(*sel, 0xFFFF)) {
	FreeLDT(*sel);
	return -1;
    }
    else
	return 0;
}

int make_selector(int sel, void *pointer32)
{
    int ds_base;

    if (!(ds_base = GetBaseAddress(get_data_seg())))
	return 0;
    if (SetBaseAddress(sel, (int)pointer32 + ds_base))
	return 0;
    else
	return sel;
}

int free_selector(int sel)
{
    return FreeLDT(sel);
}

unsigned long _convert_farp(void * addr)
{
    make_selector(sel5, addr);
    return sel5<<16;
}

void * _unconvert_farp(unsigned long fp)
{
    int ds_base;
    int p32off;

    if (!(ds_base = GetBaseAddress(get_data_seg())))
	return 0;
    if (!(p32off = GetBaseAddress(fp>>16)))
	return 0;
    return (void *)(p32off - ds_base);
}

#include <malloc.h>

unsigned int _win_alloc(unsigned long bytes)
{
    int selector;
    void *p;

    if (init_selector(&selector) == -1)
	return 0;

    if (!(p = malloc(bytes)))
	return 0;

    if (SetBaseAddress(selector, (int)p)) {
	free(p);
        return 0;
    }
    else
	return selector;
}

unsigned int _win_realloc(unsigned handle, unsigned long bytes)
{
    void *p, *q;

    /* get old pointer */
    q = (void *) GetBaseAddress(handle);

    if (!(p = realloc(q, bytes)))
	return 0;

    /* Set new base address */
    if (SetBaseAddress(handle, (int)p)) {
	free(p);
	return 0;
    }
    else
	return handle;
}

void * _win_lock(unsigned int handle)
{
    int base;
    if (!(base=GetBaseAddress(handle)))
	return (void *) 0;
    return (void *) base;
}

int _win_free(unsigned int handle)
{
    int base;
    if (!(base = GetBaseAddress(handle)))
	return -1;
    free((void *)base);
    return FreeLDT(handle);
}
