#define NOGDI
#define NOKERNEL
#include <windows.H>
#include <syscall.h>
#include <string.h>
#include "selector.h"

/* user WinMain function */
extern int WinMain(HANDLE hInst, HANDLE hPrev,LPSTR lpCmdLine, int nCmdShow);

/* callback function from syscalls.s */
extern int wnd_dlg_proc32(void);    /* callback for Windows and Dialogs */
extern int callb32_argc5(void);     /* callback for 5 args */
extern int callb32_argc4(void);     /* callback for 4 args */
extern int callb32_argc3(void);     /* callback for 3 args */
extern int callb32_argc2(void);     /* callback for 2 args */

struct {
    DWORD WinProc16;
    DWORD WinProc32;
    DWORD callback_argc5;
    DWORD callback_argc4;
    DWORD callback_argc3;
    DWORD callback_argc2;
    DWORD hInst;
    DWORD hPrevInst;
    DWORD nCmdShow;
    char lpCmdLine[256];
} prg_info_block;

ULONG callback16;

struct class {
    char *name;
    LONG (*fnct)();
} classes[20];
int classes_index=0;

int __argc;
char **__argv;
char **__env;

int sel1;
int sel2;
int sel3;
int sel4;
int sel5;

int main(int argc,char **argv,char **env)
{
    __argc = argc;
    __argv = argv;
    __env = env;
    init_selector(&sel1);
    init_selector(&sel2);
    init_selector(&sel3);
    init_selector(&sel4);
    init_selector(&sel5);

    prg_info_block.WinProc32 = (ULONG) wnd_dlg_proc32;
    prg_info_block.callback_argc5 = (ULONG) callb32_argc5;
    prg_info_block.callback_argc4 = (ULONG) callb32_argc4;
    prg_info_block.callback_argc3 = (ULONG) callb32_argc3;
    prg_info_block.callback_argc2 = (ULONG) callb32_argc2;
    SV_CALL(M_WINMAIN_INIT, (ULONG) &prg_info_block);
    callback16 = prg_info_block.WinProc16;

    return WinMain(
            prg_info_block.hInst,
            prg_info_block.hPrevInst,
            prg_info_block.lpCmdLine,
            prg_info_block.nCmdShow
            );

    free_selector(sel1);
    free_selector(sel2);
    free_selector(sel3);
    free_selector(sel4);
    free_selector(sel5);
}

HWND CreateWindow(LPSTR class, LPSTR wndname, DWORD style, short x, short y,
          short w, short h, HWND parent, HMENU menu, HANDLE hInst, LPSTR cs)
{
    STACKWORDS(2+2+2+1+1+1+1+1+1+1+2);

    {
	/* search for callback function (registered class) */
        int i;
	for (i = 0; i < classes_index; i++)
	    if (strcmp(class,classes[i].name) == 0) {
                SV_CALL(M_LOAD_WNDFNCT, (ULONG)classes[i].fnct);
                break;
            }
	/* or "listbox", "edit", "static", "mdiclient" etc. */
    }
    PUSH_LPSTR1(class);
    PUSH_LPSTR2(wndname);
    PUSH_LONG(style);
    PUSH_WORD(x);
    PUSH_WORD(y);
    PUSH_WORD(w);
    PUSH_WORD(h);
    PUSH_WORD(parent);
    PUSH_WORD(menu);
    PUSH_WORD(hInst);
    PUSH_LPSTR3(cs);
    return WINCALL(41,USER);
}

BOOL RegisterClass(LPWNDCLASS class)
{
    STACKWORDS(2);
    WNDCLASS _class;
    memcpy(&_class, class, sizeof(WNDCLASS));

    /* save class and wndproc */
    classes[classes_index].name = class->lpszClassName;
    classes[classes_index].fnct = class->lpfnWndProc;
    classes_index++;

    /* change WndProc to 16bit code WndProc */
    _class.lpfnWndProc = (LONGCALLBACK)callback16;

    /* convert menuname */
    if ((ULONG)_class.lpszMenuName > INT_RESOURCE) {
	make_selector(sel1, _class.lpszMenuName);
	_class.lpszMenuName = (char *) (sel1<<16);
    }

    /* convert classname */
    make_selector(sel2, _class.lpszClassName);
    _class.lpszClassName = (char *) (sel2<<16);


    PUSH_LPSTR3(&_class);
    return WINCALL(57,USER);
}

LONG SendMessage(HWND hWnd, WORD msg, WORD wp, LONG lp)
{
    STACKWORDS(1+1+1+2);
    MDICREATESTRUCT *pMDI = 0;

    /* user don't use CreateWindow for MDI childs */
    if (msg == WM_MDICREATE) {
	int i;

	if (!(pMDI = (MDICREATESTRUCT *) _unconvert_farp(lp)))
            return 0;

	/* search for callback function */
	for (i = 0; i < classes_index; i++)
	    if (strcmp(pMDI->szClass, classes[i].name) == 0) {
                SV_CALL(M_LOAD_WNDFNCT, (ULONG)classes[i].fnct);
                break;
            }

	/* convert title, owner */
	make_selector(sel1, pMDI->szClass);
        pMDI->szClass = (LPSTR) (sel1 << 16);
	make_selector(sel2, pMDI->szTitle);
	pMDI->szTitle = (LPSTR) (sel2 << 16);
    } /* WM_MDICREATE */

    PUSH_WORD(hWnd);
    PUSH_WORD(msg);
    PUSH_WORD(wp);
    PUSH_LONG(lp);
    return WINCALL(111,USER);
}

FARPROC MakeProcInstance(FARPROC callback, HANDLE hInst)
{
    return callback;
}

void FreeProcInstance(FARPROC thunk)
{
    return;
}

HANDLE GlobalAlloc(WORD flags, DWORD bytes)
{
    return _win_alloc(bytes);
}

HANDLE GlobalReAlloc(HANDLE oldh, DWORD bytes, WORD flags)
{
    return _win_realloc(oldh, bytes);
}

HANDLE GlobalFree(HANDLE hmem)
{
    return _win_free(hmem);
}

LPSTR GlobalLock(HANDLE hmem)
{
    return _win_lock(hmem);
}

BOOL GlobalUnlock(HANDLE hmem)
{
    return 1;
}

HANDLE LocalAlloc(WORD flags, WORD bytes)
{
    return _win_alloc((DWORD) bytes);
}

HANDLE LocalFree(HANDLE hmem)
{
    return _win_free(hmem);
}

LPSTR LocalLock(HANDLE hmem)
{
    return _win_lock(hmem);
}

BOOL LocalUnlock(HANDLE hmem)
{
    return 1;
}
