/*================================================================*/
/*                                                                */
/* File    : FILEOPEN.C                                           */
/*                                                                */
/* Purpose : Standard file-open dialog box processing. Taken from */
/*           sample code in the Windows 3.0 SDK.                  */
/*                                                                */
/* History :  modified by Rainer Schnitker			  */
/*                                                                */
/*================================================================*/

#define NOGDI             /* All GDI defines and routines           */
#define NOKERNEL	  /* All KERNEL defines and routines	    */
#define NOGDICAPMASKS     /* CC_~, LC_~, PC_~, CP_~, TC_~, RC_      */
#define NOVIRTUALKEYCODES /* VK_~				    */
#define NOSYSMETRICS	  /* SM_~				    */
#define NOMENUS 	  /* MF_~				    */
#define NOICONS 	  /* IDI_~				    */
#define NOKEYSTATES	  /* MK_~				    */
#define NOSYSCOMMANDS	  /* SC_~				    */
#define NORASTEROPS	  /* Binary and Tertiary raster ops	    */
#define NOSHOWWINDOW	  /* SW_~				    */
#define OEMRESOURCE	  /* OEM Resource values		    */
#define NOATOM		  /* Atom Manager routines		    */
#define NOCLIPBOARD	  /* Clipboard routines 		    */
#define NOCOLOR 	  /* Screen colors			    */
#define NODRAWTEXT	  /* DrawText() and DT_~		    */
#define NOMEMMGR	  /* GMEM_~, LMEM_~, GHND, LHND 	    */
#define NOMETAFILE	  /* typedef METAFILEPICT		    */
#define NOMINMAX	  /* Macros min(a,b) and max(a,b)	    */
#define NOMSG		  /* typedef MSG and associated routines    */
#define NOSCROLL	  /* SB_~ and scrolling routines	    */
#define NOSOUND 	  /* Sound driver routines		    */
#define NOTEXTMETRIC	  /* typedef TEXTMETRIC and assoc. routines */
#define NOWH		  /* SetWindowsHook and WH_~		    */
#define NOWINOFFSETS	  /* GWL_~, GCL_~, associated routines	    */
#define NOCOMM		  /* COMM driver routines		    */
#define NOKANJI 	  /* Kanji support stuff.		    */
#define NOHELP		  /* Help engine interface.		    */
#define NOPROFILER	  /* Profiler interface.		    */
#define NODEFERWINDOWPOS  /* DeferWindowPos routines		    */

#include <windows.h>
#include <string.h>
#include "stock.h"

char FileName[128];
char PathName[128];
char OpenName[128];
char DefPath[128];
char DefSpec[13] = "*.sto";
char DefExt[] = ".sto";
char str[255];


HANDLE FAR PASCAL OpenDlg(HWND hDlg, WORD message, WORD wParam, LONG lParam)
{
    HANDLE hFile;

    switch (message) {
    case WM_COMMAND:
	switch (wParam) {
	case IDC_LISTBOX:
	    switch (HIWORD(lParam)) {
	    case LBN_SELCHANGE:
		if (!DlgDirSelect(hDlg, str, IDC_LISTBOX)) {
		    SetDlgItemText(hDlg, IDC_EDIT, str);
		    SendDlgItemMessage(hDlg,
				       IDC_EDIT,
				       EM_SETSEL,
				       NULL,
				       MAKELONG(0, 0x7fff));
		} else {
		    strcat(str, DefSpec);
		    DlgDirList(hDlg, str, IDC_LISTBOX, IDC_PATH, 0x4010);
		}
		break;

	    case LBN_DBLCLK:
		goto openfile;
	    }
	    return TRUE;

	case IDOK:
	  openfile:
	    GetDlgItemText(hDlg, IDC_EDIT, OpenName, 128);
	    if (strchr(OpenName, '*') || strchr(OpenName, '?')) {
		SeparateFile(hDlg, str, DefSpec, OpenName);
		if (str[0])
		    strcpy(DefPath, str);
		ChangeDefExt(DefExt, DefSpec);
		UpdateListBox(hDlg);
		return TRUE;
	    }
	    if (!OpenName[0]) {
		MessageBox(hDlg, "No filename specified.", NULL,
			   MB_OK | MB_ICONHAND);
		return TRUE;
	    }
	    AddExt(OpenName, DefExt);

	    /* The routine to open the file would go here, and the */
	    /* handle would be returned instead of NULL.           */
	    StockFileRead((LPSTR) OpenName);

	    EndDialog(hDlg, hFile);
	    return (TRUE);

	case IDCANCEL:
	    EndDialog(hDlg, NULL);
	    return (TRUE);
	}
	break;

    case WM_INITDIALOG:
	UpdateListBox(hDlg);
	SetDlgItemText(hDlg, IDC_EDIT, DefSpec);
	SendDlgItemMessage(hDlg,
			   IDC_EDIT,
			   EM_SETSEL,
			   NULL,
			   MAKELONG(0, 0x7fff));
	SetFocus(GetDlgItem(hDlg, IDC_EDIT));
	return (FALSE);
    }
    return FALSE;
}


void UpdateListBox(HWND hDlg)
{
    strcpy(str, DefPath);
    strcat(str, DefSpec);
    DlgDirList(hDlg, str, IDC_LISTBOX, IDC_PATH, 0x4010);
    SetDlgItemText(hDlg, IDC_EDIT, DefSpec);
}


void ChangeDefExt(PSTR Ext, PSTR Name)
{
    PSTR pTptr;

    pTptr = Name;
    while (*pTptr && *pTptr != '.')
	pTptr++;
    if (*pTptr)
	if (!strchr(pTptr, '*') && !strchr(pTptr, '?'))
	    strcpy(Ext, pTptr);
}


void SeparateFile(HWND hDlg, PSTR pDestPath, PSTR pDestFileName, PSTR pSrcFileName)
{
    PSTR pTmp;
    char cTmp;

    pTmp = pSrcFileName + strlen(pSrcFileName);
    while (*pTmp != ':' && *pTmp != '\\' && pTmp > pSrcFileName)
	pTmp--;
    if (*pTmp != ':' && *pTmp != '\\') {
	strcpy(pDestFileName, pSrcFileName);
	pDestPath[0] = 0;
	return;
    }
    strcpy(pDestFileName, pTmp + 1);
    cTmp = *(pTmp + 1);
    strcpy(pDestPath, pSrcFileName);
    *(pTmp + 1) = cTmp;
    pDestPath[(pTmp - pSrcFileName) + 1] = 0;
}


void AddExt(PSTR Name, PSTR Ext)
{
    PSTR pTptr;

    pTptr = Name;
    while (*pTptr && *pTptr != '.')
	pTptr++;
    if (*pTptr != '.')
	strcat(Name, Ext);
}
